import struct

def _i16(c):
	return ord(c[0]) + (ord(c[1])<<8)

def _i32(c):
	return ord(c[0]) + (ord(c[1])<<8) + (ord(c[2])<<16) + (ord(c[3])<<24)

def _s32(n):
	return chr(n % 256) + chr((n >> 8) % 256) + chr((n >> 16) % 256) + chr((n >> 32) % 256)

def _getInt(fp):
	return struct.unpack('<i', fp.read(4))[0]#_i32(fp.read(4)) 

def _getShort(fp):
	return struct.unpack('<h', fp.read(2))[0]#_i32(fp.read(4)) 

def _write(fout, nfmt, *args):
	fmt = '<'+nfmt
	fout.write(struct.pack(fmt, *args))

def _pad(fout, size):
	fout.write('\x00'*size)

class Act:
    def save(self, outname):
		fout = file(outname, 'wb')

		fout.write('AC\x04\x02')
		_write(fout, 'h', self.nanimations)

		#fout.write(self.header)
		_pad(fout, 10)
		
		for anim in self.animations:
			nframes, frames = anim

			_write(fout, 'i', nframes)
			for frame in frames:

				nsubframes, subframes, isextrainfo, extraX, extraY = frame

				_pad(fout, 32)
				
				_write(fout, 'i', nsubframes)

				for subframe in subframes:
					offsetX, offsetY, image, direction, (cred, cgreen, cblue, calpha), scaleX, scaleY, withsize, sizeX, sizeY, rgba = subframe
					_write(fout, 'iiiiBBBBffii', offsetX, offsetY,
					       image, direction, cred, cgreen, cblue, calpha,
					       scaleX, scaleY, 0, rgba)

				_write(fout, 'ii', -1, isextrainfo)

				for ex in range(isextrainfo):
					_write(fout, 'iiii', 0, extraX, extraY, 0)

		_write(fout, 'i', 0)

		for timescale in self.timescale:
			_write(fout, 'f', timescale)
		fout.close()
	
    def __init__(self, filename,verbose=0, empty=0):
	    if empty:
		    return
            fp = file(filename, 'rb')
            fp.seek(2, 1)
            self.version = ord(fp.read(1))
            if verbose:
                print 'Version :'+str(self.version)
            fp.seek(1, 1)
            self.nanimations = _getShort(fp)
            if verbose:
                print 'Animations : '+str(self.nanimations)

	    self.header = fp.read(10)
	    
            self.animations = []
            for i in range(self.nanimations):
                nframes = _getInt(fp)
                frames = []
                if verbose:
                    print 'Animation %d : %d frames' % (i, nframes)
                    
                for j in range(nframes):
		    fp.seek(32, 1)
		
                    nsubframes = _getInt(fp)
                    subframes = []

                    for k in range(nsubframes):
                        if verbose:
                            print '%X::%d %d' % (fp.tell(), j, k)
                        offsetX, offsetY, image, direction, cred, cgreen, cblue, calpha = struct.unpack(
                            '<iiiiBBBB', fp.read(20))

                        if self.version >= 2:
                                scaleX = struct.unpack('<f', fp.read(4))[0]
                        else:
                                scaleX = 1.0

                        if self.version >= 4:
                            scaleY = struct.unpack('<f', fp.read(4))[0]
                        else:
                            scaleY = scaleX

                        if self.version >= 2:
                                todo, rgba = struct.unpack('<ii', fp.read(8))
                        else:
				todo = 0
                                rgba = 0
                                fp.seek(12, 1)

                        if self.version >= 5:
                            withsize = 1
                            sizeX, sizeY = struct.unpack('<ii', fp.read(8))
                        else:
                            withsize, sizeX, sizeY = 0, -1, -1

                        subframes.append(
                            (offsetX, offsetY, image, direction,
                            (cred, cgreen, cblue, calpha), scaleX, scaleY,
			     withsize, sizeX, sizeY, rgba) )

		    if self.version >= 2:
			    fp.seek(4, 1)
			    isextrainfo = _getInt(fp)
			    extraX, extraY = 99999, 99999
		    else:
			    fp.seek(4, 1)
			    isextrainfo = 0
			    extraX, extraY = 99999, 99999

		    for ex in range(isextrainfo):
			    fp.seek(4, 1)
			    extraX, extraY = struct.unpack('<ii', fp.read(8))
			    fp.seek(4, 1)
			    
                    frames.append(
                        (nsubframes, subframes, isextrainfo, extraX, extraY) )

                self.animations.append( (nframes, frames) )
            nsounds = _getInt(fp)
            fp.seek(40*nsounds, 1)

	    self.timescale = []
	    str = fp.read(4)
	    while str != '':
		    self.timescale.append(struct.unpack('<f', str)[0])
		    str = fp.read(4)
		    
            fp.close()

    def getNFrames(self, numAnimation):
        nframes, frames = self.animations[numAnimation]
	return nframes
                        
    def getOffset(self, numAnimation, numFrame, numSubframe):
        nframes, frames = self.animations[numAnimation]
        if numFrame >= nframes:
		nFrame = 0
	else:
		nFrame = numFrame
        nsubframes, subframes, isextrainfo, extraX, extraY = frames[nFrame]
        offsetX, offsetY, image, direction, (cred, cgreen, cblue, calpha), scaleX, scaleY, withsize, sizeX, sizeY = subframes[numSubframe]
        return offsetX, offsetY

    def getExtra(self, numAnimation, numFrame):
        nframes, frames = self.animations[numAnimation]
        if numFrame >= nframes:
		nFrame = 0
	else:
		nFrame = numFrame
        nsubframes, subframes, isextrainfo, extraX, extraY = frames[nFrame]
        return extraX, extraY

    def getNSubframes(self, numAnimation, numFrame):
        nframes, frames = self.animations[numAnimation]
        if numFrame >= nframes:
		nFrame = 0
	else:
		nFrame = numFrame
        nsubframes, subframes, isextrainfo, extraX, extraY = frames[nFrame]
        return nsubframes

    def getImage(self, numAnimation, numFrame, numSubframe):
        nframes, frames = self.animations[numAnimation]
        if numFrame >= nframes:
		nFrame = 0
	else:
		nFrame = numFrame
        nsubframes, subframes, isextrainfo, extraX, extraY = frames[nFrame]
        offsetX, offsetY, image, direction, (cred, cgreen, cblue, calpha), scaleX, scaleY, withsize, sizeX, sizeY = subframes[numSubframe]
        return image

    def getDirection(self, numAnimation, numFrame, numSubframe):
        nframes, frames = self.animations[numAnimation]
        if numFrame >= nframes:
		nFrame = 0
	else:
		nFrame = numFrame
        nsubframes, subframes, isextrainfo, extraX, extraY = frames[nFrame]
        offsetX, offsetY, image, direction, (cred, cgreen, cblue, calpha), scaleX, scaleY, withsize, sizeX, sizeY = subframes[numSubframe]
        return direction

    def getColor(self, numAnimation, numFrame, numSubframe):
        nframes, frames = self.animations[numAnimation]
        if numFrame >= nframes:
		nFrame = 0
	else:
		nFrame = numFrame
        nsubframes, subframes, isextrainfo, extraX, extraY = frames[nFrame]
        offsetX, offsetY, image, direction, (cred, cgreen, cblue, calpha), scaleX, scaleY, withsize, sizeX, sizeY = subframes[numSubframe]
        return cred, cgreen, cblue, calpha

    def getScale(self, numAnimation, numFrame, numSubframe):
        nframes, frames = self.animations[numAnimation]
        if numFrame >= nframes:
		nFrame = 0
	else:
		nFrame = numFrame
        nsubframes, subframes, isextrainfo, extraX, extraY = frames[nFrame]
        offsetX, offsetY, image, direction, (cred, cgreen, cblue, calpha), scaleX, scaleY, withsize, sizeX, sizeY = subframes[numSubframe]
        return scaleX, scaleY

    def getSize(self, numAnimation, numFrame, numSubframe):
        nframes, frames = self.animations[numAnimation]
        if numFrame >= nframes:
		nFrame = 0
	else:
		nFrame = numFrame
        nsubframes, subframes, isextrainfo, extraX, extraY = frames[nFrame]
        offsetX, offsetY, image, direction, (cred, cgreen, cblue, calpha), scaleX, scaleY, withsize, sizeX, sizeY = subframes[numSubframe]
        return sizeX, sizeY

    def getTotalFrames(self):
	    tf = 0
	    for i in range(self.nanimations):
		    tf += self.getNFrames(i)
	    return tf
