import os
import grfio
from spr import *
from act import *
from sprite import *

from wxPython.wx import *
from spracttosprite import SprAct2Sprite

name = 'Open from a GRF data file'
type = 'importer'
doc = """rogrfOpener plugin by deufeufeu
    Firstly, when executed, this plugin will let you choose a
    grf file like the data.grf or sdata.grf that lies in your
    RO base directory.
    Then it will display the list of files from the choosen grf
    and let you choose a .spr or a .act file. If you choose a .spr, like
    banana.spr it will load the sprite using banana.spr and banana.act,
    it's the same if you choose a .act file. So be careful to choose a
    spr/act with existing counterpart.

    Repeat action :
        When u use the repeat last command after having used this plugin
        it'll let you directly choose files from the grf files previously
        choosed."""

_repeat = ''

def getSprite(parent, repeat=0):
    global _repeat
    
    if not repeat or not _repeat:
        dlg = wxFileDialog(parent, 'Choose archive', os.getcwd(),
                           '', 'GRF data file (*.grf)|*.grf', wxOPEN)
        if dlg.ShowModal() == wxID_OK:
            filename = dlg.GetPath()
            _repeat = filename

            fp = file('grf-files.txt', 'w')
            fp.write('data:'+filename)
            fp.close()

            grfio.grfio_init()
            return _do(parent)
        dlg.Destroy()
    else:
        grfio.grfio_init()
        return _do(parent)
    return None

def _do(parent):
    grfdlg = xzvfGrfContent(parent)
    if grfdlg.ShowModal() == wxID_OK:
        num = grfdlg.getNum()
        grfdlg.Destroy()
        name = grfio.grfio_getentryname(num)

        grflist = []
        for i in range(grfio.grfio_numentrys()):
            grflist.append(grfio.grfio_getentryname(i))
            
        if name[-3:] == 'spr':
            sprname = name
            actname = name[:-3]+'act'
            if actname not in grflist:
                print 'Non valid spr file '+name
                return None
        elif name[-3:] == 'act':
            actname = name
            sprname = name[:-3]+'spr'
            if sprname not in grflist:
                print 'Non valid act file '+name
                return None
        else:
            print 'Non Sprite related file'
            return None
    else:
        return None
    grfdlg.Destroy()

    grfio.grfio_extractFileFromName(sprname, 'temp.spr')
    grfio.grfio_extractFileFromName(actname, 'temp.act')
    spr = Spr('temp.spr')
    act = Act('temp.act')
    sprite = SprAct2Sprite(spr, act) 
    os.remove('temp.spr')
    os.remove('temp.act')
    return sprite
    
class xzvfGrfVirtualList(wxListCtrl):
    def __init__(self, parent, id):
        wxListCtrl.__init__(self, parent, id, wxPoint(10, 10), size=wxSize(480, 300),
                            style = wxLC_REPORT | wxLC_VIRTUAL | wxSUNKEN_BORDER)

        self.InsertColumn(0, 'Name')
        self.SetColumnWidth(0, 450)
        
        self.SetItemCount(grfio.grfio_numentrys())
        
        EVT_LIST_ITEM_SELECTED(self, self.GetId(), self.OnItemSelected)
     #   EVT_LIST_ITEM_ACTIVATED(self, self.GetId(), self.OnItemSelected)
        self.n = 0

    def OnGetItemText(self, item, col):
        return grfio.grfio_getentryname(item)

    def OnGetItemImage(self, item):
        return -1

    def OnItemSelected(self, event):
        temp = event.m_itemIndex
        self.n = temp

class xzvfGrfContent(wxDialog):
    def __init__(self, parent, size=wxSize(500, 400)):
        wxDialog.__init__(self, parent, -1, 'Choose file', pos=wxDefaultPosition,
                          size=size, style=wxDEFAULT_DIALOG_STYLE)
        self.sizer = wxBoxSizer(wxVERTICAL)

        id = wxNewId()
        self.liste = xzvfGrfVirtualList(self, id)

        btn = wxButton(self, wxID_OK, ' OK ', wxPoint(10, 320))
        btn.SetDefault()

        self.SetAutoLayout(1)
        self.n = -1
        
    def getNum(self):
        return self.liste.n
