""" This is the class that handles the graphics for sprites
    and generate texture"""
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from OpenGL.GL.ARB.texture_compression import *

IMAGE_RGBA = 0x01
IMAGE_INDEX = 0x02
	
def _bestsize(dim):
    boundary = min(glGetInteger(GL_MAX_TEXTURE_SIZE), dim)
    
    test = 1
    while test < boundary:
        test = test * 2
    return test

class Sprite:
    def initDrawing(self):
        zero = '\x00'
        if self.ntextures == 1:
            self.textures = [glGenTextures(1)]
        else:
            self.textures = glGenTextures(self.ntextures)

        self.texsize = []

        glPixelMapfv(GL_PIXEL_MAP_I_TO_R, [0.0]+self.palR[1:])
        glPixelMapfv(GL_PIXEL_MAP_I_TO_G, [0.0]+self.palG[1:])
        glPixelMapfv(GL_PIXEL_MAP_I_TO_B, [0.0]+self.palB[1:])
        glPixelMapfv(GL_PIXEL_MAP_I_TO_A, [0.0]+[1.0]*255)

        for i in range(self.ntextures):
            if i < self.nImages:
                indexed = 1
                sx, sy, data = self.images[i]
            else:
                indexed = 0
                sx, sy, data = self.rgbaImages[i-self.nImages]
                
            nsx, nsy = _bestsize(sx), _bestsize(sy)
            glBindTexture(GL_TEXTURE_2D, self.textures[i])
            glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
                                 nsx, nsy, 0, GL_RGBA, GL_UNSIGNED_BYTE,
                                 zero*nsx*nsy*4)


            self.texsize.append((nsx, nsy))

            if indexed:
                glPixelStorei(GL_UNPACK_ALIGNMENT, 1)
                glTexSubImage2D(GL_TEXTURE_2D, 0, (nsx-sx)/2, (nsy-sy)/2,
                                           sx, sy, GL_COLOR_INDEX, GL_UNSIGNED_BYTE,
                                           data)
            else:
                glPixelStorei(GL_UNPACK_ALIGNMENT, 4)
                glTexSubImage2D(GL_TEXTURE_2D, 0, (nsx-sx)/2, (nsy-sy)/2,
                                           sx, sy, GL_RGBA, GL_UNSIGNED_BYTE,
                                           data)

            glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
            glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
            glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
            glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)

            glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE)

    def getNFrames(self, nanim):
        frames = self.animations[nanim]
        return len(frames)

    def draw(self, nanim, nframe):        
        frames = self.animations[nanim]
        frame = frames[nframe]

        for subframe in frame['subframes']:
            x = subframe['x']
            y = subframe['y']
            imageNum = subframe['imageNum']
            direction = subframe['direction']
            cr = subframe['red']
            cg = subframe['green']
            cb = subframe['blue']
            ca = subframe['alpha']
            scaleX = subframe['scaleX']
            scaleY = subframe['scaleY']
            rgba = subframe['rgba']

            
            sizeX, sizeY = self.texsize[imageNum]

            #imagetype, sizeX, sizeY, data = self.images[imageNum]
                
            if imageNum < 0 or (sizeX <= 0 and sizeY <= 0):
                continue

            if rgba:
                tex = self.textures[imageNum]+self.nImages
            else:
                tex = self.textures[imageNum]
            
            glBindTexture(GL_TEXTURE_2D, tex)

            glPushMatrix()

            glScalef(0.02, 0.02, 0.0)
            glTranslatef(float(x)*2.0, float(-y)*2.0, 0.0)
            glScalef(scaleX*float(sizeX), scaleY*float(sizeY), 0.0)

            glBegin(GL_QUADS)

            glColor4f(cr/255.0, cg/255.0, cb/255.0, ca/255.0)

            if not direction:
                u1, v1 = 1.0, 0.0
                u2, v2 = 0.0, 0.0
                u3, v3 = 0.0, 1.0
                u4, v4 = 1.0, 1.0
            else:
                u1, v1 = 0.0, 0.0
                u2, v2 = 1.0, 0.0
                u3, v3 = 1.0, 1.0
                u4, v4 = 0.0, 1.0
                

            glTexCoord2f(u1, v1); glVertex3f(-1.0, -1.0,  0.0)
            glTexCoord2f(u2, v2); glVertex3f( 1.0, -1.0,  0.0)
            glTexCoord2f(u3, v3); glVertex3f( 1.0,  1.0,  0.0)
            glTexCoord2f(u4, v4); glVertex3f(-1.0,  1.0,  0.0)
                         
            glEnd()
            
            glPopMatrix()

if __name__ == "__main__":
    pass
