from wxPython.wx import *
import os
import Image
import struct

type = 'tool'
name = 'Create a spr from a list of images'
doc = """Spr Creator by deufeufeu

This tool will let you  create a spr by selecting indexed
bitmaps and then rgba images."""

def _write(fout, nfmt, *args):
	fmt = '<'+nfmt
	fout.write(struct.pack(fmt, *args))

def _pad(fout, size):
	fout.write('\x00'*size)

def _error(parent, text):
    dlg = wxMessageDialog(parent, text,
                          'Error', wxOK | wxICON_ERROR)
    dlg.ShowModal()

def execute(parent, repeat=0):
    iImages = []
    dlg = wxFileDialog(parent, 'Choose indexed bitmap files', os.getcwd(), '',
                       'Bitmap (*.bmp)|*.bmp',
                       wxOPEN | wxMULTIPLE)
    if dlg.ShowModal() == wxID_OK:
        paths = dlg.GetPaths()
        for path in paths:
            image = Image.open(path)
            if image.mode != 'P':
                _error(parent, 'One of the selected images is not in indexed format (256 colors)\nExiting')
                return
            iImages.append(image)

    rgbaImages = []
    dlg = wxFileDialog(parent, 'Choose image files with alpha', os.getcwd(), '',
                       'All files (*.*)|*.*',
                       wxOPEN | wxMULTIPLE)
    if dlg.ShowModal() == wxID_OK:
        paths = dlg.GetPaths()
        for path in paths:
            try:
                image = Image.open(path)
                if image.mode != 'RGBA':
                    image = image.convert('RGBA')
                rgbaImages.append(image)
            except IOError:
                _error(parent, 'One of the selected files is not an image.\nExiting')
                return

    if len(iImages)+len(rgbaImages) == 0:
        _error(parent, 'You have to select at least one image\nExiting')
        return

    dlg = wxFileDialog(parent, 'Save to an spr file', os.getcwd(),
                       '', 'Spr File (*.spr)|*.spr',
                       wxSAVE | wxOVERWRITE_PROMPT)
    if dlg.ShowModal() == wxID_OK:
        sprname = dlg.GetPath()
    else:
        return

    fp = file(sprname, 'wb')

    fp.write('SP\x01\x02')

    _write(fp, 'hh', len(iImages), len(rgbaImages))

    pal = None
    for image in iImages:
        imgstr = image.tostring()
        rleencoded = ''
        i = 0
        count = 0
        for char in imgstr:
            if ord(char) == 0:
                count += 1
            else:
                if count != 0:
                    rleencoded += '\x00'+chr(count)
                    count = 0
                rleencoded += char
        w, h = image.size
        _write(fp, 'hhh', w, h, len(rleencoded))
        fp.write(rleencoded)
        
        if not pal:
            imgpal = image.palette
            mode = imgpal.mode
            data = imgpal.palette

            ncol = len(data)/3
            pal = '\x00'*4
            for i in range(1, 256):
                if i >= ncol:
                    pal += '\x00'*4
                else:
               #     if mode == 'RGB':
               #         pal += data[3*i:3*(i+1)]+'\xff'
               #     else:
                        pal += data[3*i+2] + data[3*i+1] + data[3*i] + '\xff'

    for image in rgbaImages:
        w, h = image.size
        _write(fp, 'hh', w, h)
        fp.write(image.tostring())

    fp.write(pal)
            
    fp.close()
